@@ -51,11 +51,16 @@
 #define HAL_DBUS_DEVICE_INTERFACE               "org.freedesktop.Hal.Device"
 #define SEAT_PCI_DEVICE_CLASS                   3
 
+#define MAX_DISPLAY_FAILURES 5
+
 struct GdmLocalDisplayFactoryPrivate
 {
         DBusGConnection *connection;
         DBusGProxy      *proxy;
         GHashTable      *displays;
+
+        /* FIXME: this needs to be per seat? */
+        guint            num_failures;
 };
 
 enum {
@@ -66,6 +71,8 @@ static void     gdm_local_display_factory_class_init    (GdmLocalDisplayFactoryC
 static void     gdm_local_display_factory_init          (GdmLocalDisplayFactory      *factory);
 static void     gdm_local_display_factory_finalize      (GObject                     *object);
 
+static GdmDisplay *create_display                       (GdmLocalDisplayFactory      *factory);
+
 static gpointer local_display_factory_object = NULL;
 
 G_DEFINE_TYPE (GdmLocalDisplayFactory, gdm_local_display_factory, GDM_TYPE_DISPLAY_FACTORY)
@@ -128,7 +135,7 @@ take_next_display_number (GdmLocalDisplayFactory *factory)
 
         g_debug ("GdmLocalDisplayFactory: Found the following X displays:");
         for (l = list; l != NULL; l = l->next) {
-                g_debug ("%u", GPOINTER_TO_UINT (l->data));
+                g_debug ("GdmLocalDisplayFactory: %u", GPOINTER_TO_UINT (l->data));
         }
 
         for (l = list; l != NULL; l = l->next) {
@@ -159,8 +166,7 @@ static void
 on_display_disposed (GdmLocalDisplayFactory *factory,
                      GdmDisplay             *display)
 {
-        /* remove the display number from accounting */
-
+        g_debug ("GdmLocalDisplayFactory: Display %p disposed", display);
 }
 
 static void
@@ -205,11 +211,6 @@ gdm_local_display_factory_create_transient_display (GdmLocalDisplayFactory *fact
 
         display = gdm_transient_display_new (num);
 
-        if (! gdm_display_create_authority (display)) {
-                display = NULL;
-                goto out;
-        }
-
         /* FIXME: don't hardcode seat1? */
         g_object_set (display, "seat-id", CK_SEAT1_PATH, NULL);
 
@@ -257,11 +258,6 @@ gdm_local_display_factory_create_product_display (GdmLocalDisplayFactory *factor
 
         display = gdm_product_display_new (num, relay_address);
 
-        if (! gdm_display_create_authority (display)) {
-                display = NULL;
-                goto out;
-        }
-
         /* FIXME: don't hardcode seat1? */
         g_object_set (display, "seat-id", CK_SEAT1_PATH, NULL);
 
@@ -285,6 +281,57 @@ gdm_local_display_factory_create_product_display (GdmLocalDisplayFactory *factor
         return ret;
 }
 
+static void
+on_static_display_status_changed (GdmDisplay             *display,
+                                  GParamSpec             *arg1,
+                                  GdmLocalDisplayFactory *factory)
+{
+        int              status;
+        GdmDisplayStore *store;
+        int              num;
+
+        num = -1;
+        gdm_display_get_x11_display_number (display, &num, NULL);
+        g_assert (num != -1);
+
+        store = gdm_display_factory_get_display_store (GDM_DISPLAY_FACTORY (factory));
+
+        status = gdm_display_get_status (display);
+
+        g_debug ("GdmLocalDisplayFactory: static display status changed: %d", status);
+        switch (status) {
+        case GDM_DISPLAY_FINISHED:
+                /* remove the display number from factory->priv->displays
+                   so that it may be reused */
+                g_hash_table_remove (factory->priv->displays, GUINT_TO_POINTER (num));
+                gdm_display_store_remove (store, display);
+                /* reset num failures */
+                factory->priv->num_failures = 0;
+                create_display (factory);
+                break;
+        case GDM_DISPLAY_FAILED:
+                /* leave the display number in factory->priv->displays
+                   so that it doesn't get reused */
+                gdm_display_store_remove (store, display);
+                factory->priv->num_failures++;
+                if (factory->priv->num_failures > MAX_DISPLAY_FAILURES) {
+                        /* oh shit */
+                        g_critical ("GdmLocalDisplayFactory: maximum number of displays failures reached: check Xorg.log for errors");
+                        exit (1);
+                }
+
+                create_display (factory);
+                break;
+        case GDM_DISPLAY_UNMANAGED:
+                break;
+        case GDM_DISPLAY_MANAGED:
+                break;
+        default:
+                g_assert_not_reached ();
+                break;
+        }
+}
+
 static GdmDisplay *
 create_display (GdmLocalDisplayFactory *factory)
 {
@@ -303,16 +350,14 @@ create_display (GdmLocalDisplayFactory *factory)
                 return NULL;
         }
 
-        if (! gdm_display_create_authority (display)) {
-                g_warning ("Unable to set up access control for display %d",
-                           num);
-                g_object_unref (display);
-                return NULL;
-        }
-
         /* FIXME: don't hardcode seat1? */
         g_object_set (display, "seat-id", CK_SEAT1_PATH, NULL);
 
+        g_signal_connect (display,
+                          "notify::status",
+                          G_CALLBACK (on_static_display_status_changed),
+                          factory);
+
         store_display (factory, num, display);
 
         /* let store own the ref */
@@ -438,10 +483,6 @@ gdm_local_display_factory_set_property (GObject       *object,
                                         const GValue  *value,
                                         GParamSpec    *pspec)
 {
-        GdmLocalDisplayFactory *self;
-
-        self = GDM_LOCAL_DISPLAY_FACTORY (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -455,10 +496,6 @@ gdm_local_display_factory_get_property (GObject    *object,
                                         GValue     *value,
                                         GParamSpec *pspec)
 {
-        GdmLocalDisplayFactory *self;
-
-        self = GDM_LOCAL_DISPLAY_FACTORY (object);
-
         switch (prop_id) {
         default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -515,11 +552,8 @@ gdm_local_display_factory_constructor (GType                  type,
                                        GObjectConstructParam *construct_properties)
 {
         GdmLocalDisplayFactory      *factory;
-        GdmLocalDisplayFactoryClass *klass;
         gboolean                     res;
 
-        klass = GDM_LOCAL_DISPLAY_FACTORY_CLASS (g_type_class_peek (GDM_TYPE_LOCAL_DISPLAY_FACTORY));
-
         factory = GDM_LOCAL_DISPLAY_FACTORY (G_OBJECT_CLASS (gdm_local_display_factory_parent_class)->constructor (type,
                                                                                                                    n_construct_properties,
                                                                                                                    construct_properties));
